@@ -31,11 +31,20 @@
 #include "gth-media-viewer-page.h"
 
 
-#define GTH_MEDIA_VIEWER_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTH_TYPE_MEDIA_VIEWER_PAGE, GthMediaViewerPagePrivate))
 #define GET_WIDGET(x) (_gtk_builder_get_widget (self->priv->builder, (x)))
 #define PROGRESS_DELAY 500
 
 
+static void gth_viewer_page_interface_init (GthViewerPageInterface *iface);
+
+
+G_DEFINE_TYPE_WITH_CODE (GthMediaViewerPage,
+			 gth_media_viewer_page,
+			 G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (GTH_TYPE_VIEWER_PAGE,
+					 	gth_viewer_page_interface_init))
+
+
 struct _GthMediaViewerPagePrivate {
 	GthBrowser     *browser;
 	GtkActionGroup *actions;
@@ -45,6 +54,7 @@ struct _GthMediaViewerPagePrivate {
 	GtkBuilder     *builder;
 	GtkWidget      *area;
 	GtkWidget      *area_box;
+	gboolean        visible;
 	gboolean        playing;
 	gboolean        paused;
 	gdouble         last_volume;
@@ -58,6 +68,7 @@ struct _GthMediaViewerPagePrivate {
 	gdouble         rate;
 	GtkWidget      *mediabar;
 	GtkWidget      *fullscreen_toolbar;
+	gulong          video_window_xid;
 	gboolean        xwin_assigned;
 	GdkPixbuf      *icon;
 	PangoLayout    *caption_layout;
@@ -69,7 +80,6 @@ struct _GthMediaViewerPagePrivate {
 };
 
 
-static gpointer gth_media_viewer_page_parent_class = NULL;
 static double default_rates[] = { 0.03, 0.06, 0.12, 0.25, 0.33, 0.50, 0.66, 1.0, 1.50, 2.0, 3.0, 4.0, 8.0, 16.0, 32.0 };
 
 
@@ -137,6 +147,7 @@ video_area_realize_cb (GtkWidget *widget,
 {
 	GthMediaViewerPage *self = user_data;
 
+	self->priv->video_window_xid = GDK_WINDOW_XID (gtk_widget_get_window (widget));
 	self->priv->cursor = gdk_cursor_new (GDK_LEFT_PTR);
 	self->priv->cursor_void = gdk_cursor_new_for_display (gtk_widget_get_display (self->priv->area), GDK_BLANK_CURSOR);
 	if (self->priv->cursor_visible)
@@ -156,13 +167,16 @@ video_area_unrealize_cb (GtkWidget *widget,
 {
 	GthMediaViewerPage *self = user_data;
 
+	self->priv->video_window_xid = 0;
+	self->priv->xwin_assigned = FALSE;
+
 	if (self->priv->cursor) {
-		gdk_cursor_unref (self->priv->cursor);
+		g_object_unref (self->priv->cursor);
 		self->priv->cursor = NULL;
 	}
 
 	if (self->priv->cursor_void) {
-		gdk_cursor_unref (self->priv->cursor_void);
+		g_object_unref (self->priv->cursor_void);
 		self->priv->cursor_void = NULL;
 	}
 
@@ -172,23 +186,19 @@ video_area_unrealize_cb (GtkWidget *widget,
 
 
 static gboolean
-video_area_expose_event_cb (GtkWidget      *widget,
-			    GdkEventExpose *event,
-			    gpointer        user_data)
+video_area_draw_cb (GtkWidget *widget,
+		    cairo_t   *cr,
+		    gpointer   user_data)
 {
 	GthMediaViewerPage *self = user_data;
 	GtkAllocation       allocation;
-	GtkStyle           *style;
-	cairo_t            *cr;
-
-	if (event->count > 0)
-		return FALSE;
+	GtkStyleContext    *style_context;
 
 	if (self->priv->xwin_assigned && self->priv->has_video)
 		return FALSE;
 
 	gtk_widget_get_allocation (widget, &allocation);
-	style = gtk_widget_get_style (widget);
+	style_context = gtk_widget_get_style_context (widget);
 
 	if (self->priv->icon == NULL) {
 		char  *type;
@@ -211,14 +221,8 @@ video_area_expose_event_cb (GtkWidget      *widget,
 		g_free (type);
 	}
 
-	cr = gdk_cairo_create (gtk_widget_get_window (widget));
-	gdk_cairo_region (cr, event->region);
-	cairo_clip (cr);
-	if (self->priv->has_video)
-		cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
-	else
-		gdk_cairo_set_source_color (cr, &style->text[gtk_widget_get_state (widget)]);
-	gdk_cairo_region (cr, event->region);
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+	cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
 	cairo_fill (cr);
 
 	if (self->priv->icon != NULL) {
@@ -246,13 +250,12 @@ video_area_expose_event_cb (GtkWidget      *widget,
 		cairo_fill (cr);
 
 		cairo_move_to (cr, x, y);
+		pango_layout_set_font_description (self->priv->caption_layout, gtk_style_context_get_font (style_context, gtk_widget_get_state (widget)));
 		pango_cairo_layout_path (cr, self->priv->caption_layout);
-		gdk_cairo_set_source_color (cr, &style->base[gtk_widget_get_state (widget)]);
+		cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 		cairo_fill (cr);
 	}
 
-	cairo_destroy (cr);
-
 	return TRUE;
 }
 
@@ -263,7 +266,7 @@ video_area_button_press_cb (GtkWidget          *widget,
 			    GthMediaViewerPage *self)
 {
 	if ((event->type == GDK_BUTTON_PRESS) && (event->button == 1) ) {
-		gtk_button_clicked (GTK_BUTTON (GET_WIDGET ("button_play")));
+		gtk_button_clicked (GTK_BUTTON (GET_WIDGET ("play_button")));
 		return TRUE;
 	}
 
@@ -342,9 +345,9 @@ update_current_position_bar (GthMediaViewerPage *self,
 		*/
 
         	if (update_progressbar) {
-			g_signal_handlers_block_by_func(GET_WIDGET ("adjustment_position"), position_value_changed_cb, self);
-			gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("adjustment_position")), (self->priv->duration > 0) ? ((double) current_value / self->priv->duration) * 100.0 : 0.0);
-			g_signal_handlers_unblock_by_func(GET_WIDGET ("adjustment_position"), position_value_changed_cb, self);
+			g_signal_handlers_block_by_func(GET_WIDGET ("position_adjustment"), position_value_changed_cb, self);
+			gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("position_adjustment")), (self->priv->duration > 0) ? ((double) current_value / self->priv->duration) * 100.0 : 0.0);
+			g_signal_handlers_unblock_by_func(GET_WIDGET ("position_adjustment"), position_value_changed_cb, self);
         	}
 
         	s = _g_format_duration_for_display (GST_TIME_AS_MSECONDS (current_value));
@@ -384,7 +387,7 @@ position_value_changed_cb (GtkAdjustment *adjustment,
 
 
 static char *
-hscale_volume_format_value_cb (GtkScale *scale,
+volume_scale_format_value_cb (GtkScale *scale,
 			       double    value,
 			       gpointer  user_data)
 {
@@ -393,7 +396,7 @@ hscale_volume_format_value_cb (GtkScale *scale,
 
 
 static gboolean
-hscale_position_change_value_cb (GtkRange      *range,
+position_scale_change_value_cb (GtkRange      *range,
 				 GtkScrollType  scroll,
 				 gdouble        value,
 				 gpointer       user_data)
@@ -410,12 +413,37 @@ hscale_position_change_value_cb (GtkRange      *range,
 
 
 static gboolean
-hscale_position_button_release_event_cb (GtkWidget      *widget,
+position_scale_button_press_event_cb (GtkWidget      *widget,
+				      GdkEventButton *event,
+				      gpointer        user_data)
+{
+	/* In a video player when clicking on the progress bar the user expect
+	 * to jump to the specified position directly instead of scrolling one
+	 * page up or down as happens by default in Gtk+.  The button 2
+	 * behavior is what we want by default for button 1. */
+
+	if (event->button == 1)
+		event->button = 2;
+	else if (event->button == 2)
+		event->button = 1;
+
+	return FALSE;
+}
+
+
+static gboolean
+position_scale_button_release_event_cb (GtkWidget      *widget,
 					 GdkEventButton *event,
 					 gpointer        user_data)
 {
 	GthMediaViewerPage *self = user_data;
 
+	/* Swap button 1 and 2 behaviors, see position_scale_button_press_event_cb */
+	if (event->button == 1)
+		event->button = 2;
+	else if (event->button == 2)
+		event->button = 1;
+
 	if (self->priv->playing)
 		self->priv->block_next_jump = TRUE;
 
@@ -453,7 +481,7 @@ update_player_rate (GthMediaViewerPage *self)
 	if (! self->priv->playing)
 		return;
 
-	current_value = (gint64) (gtk_adjustment_get_value (GTK_ADJUSTMENT (GET_WIDGET ("adjustment_position"))) / 100.0 * self->priv->duration);
+	current_value = (gint64) (gtk_adjustment_get_value (GTK_ADJUSTMENT (GET_WIDGET ("position_adjustment"))) / 100.0 * self->priv->duration);
 	if (! gst_element_seek (self->priv->playbin,
 				self->priv->rate,
 			        GST_FORMAT_TIME,
@@ -469,7 +497,7 @@ update_player_rate (GthMediaViewerPage *self)
 
 
 static void
-button_play_clicked_cb (GtkButton *button,
+play_button_clicked_cb (GtkButton *button,
 			gpointer   user_data)
 {
 	GthMediaViewerPage *self = user_data;
@@ -491,7 +519,7 @@ button_play_clicked_cb (GtkButton *button,
 		else {
 			gint64 current_value;
 
-			current_value = (gint64) (gtk_adjustment_get_value (GTK_ADJUSTMENT (GET_WIDGET ("adjustment_position"))) / 100.0 * self->priv->duration);
+			current_value = (gint64) (gtk_adjustment_get_value (GTK_ADJUSTMENT (GET_WIDGET ("position_adjustment"))) / 100.0 * self->priv->duration);
 			gst_element_seek (self->priv->playbin,
 					  self->priv->rate,
 					  GST_FORMAT_TIME,
@@ -509,7 +537,7 @@ button_play_clicked_cb (GtkButton *button,
 
 
 static void
-togglebutton_volume_toggled_cb (GtkToggleButton *button,
+volume_togglebutton_toggled_cb (GtkToggleButton *button,
 				gpointer         user_data)
 {
 	GthMediaViewerPage *self = user_data;
@@ -530,7 +558,7 @@ static int
 get_nearest_rate (double rate)
 {
 	int    min_idx = -1;
-	double min_delta;
+	double min_delta = 0;
 	int    i;
 
 	for (i = 0; i < G_N_ELEMENTS (default_rates); i++) {
@@ -548,7 +576,7 @@ get_nearest_rate (double rate)
 
 
 static void
-button_play_slower_clicked_cb (GtkButton *button,
+play_slower_button_clicked_cb (GtkButton *button,
 			       gpointer   user_data)
 {
 	GthMediaViewerPage *self = user_data;
@@ -565,7 +593,7 @@ button_play_slower_clicked_cb (GtkButton *button,
 
 
 static void
-button_play_faster_clicked_cb (GtkButton *button,
+play_faster_button_clicked_cb (GtkButton *button,
 			       gpointer   user_data)
 {
 	GthMediaViewerPage *self = user_data;
@@ -590,18 +618,19 @@ update_volume_from_playbin (GthMediaViewerPage *self)
 		return;
 
 	g_object_get (self->priv->playbin, "volume", &volume, NULL);
-	if (volume == 0.0)
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("togglebutton_volume_image")), "audio-volume-muted", GTK_ICON_SIZE_BUTTON);
-	else if (volume < 3.3)
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("togglebutton_volume_image")), "audio-volume-low", GTK_ICON_SIZE_BUTTON);
-	else if (volume < 6.6)
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("togglebutton_volume_image")), "audio-volume-medium", GTK_ICON_SIZE_BUTTON);
-	else
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("togglebutton_volume_image")), "audio-volume-high", GTK_ICON_SIZE_BUTTON);
 
-	g_signal_handlers_block_by_func(GET_WIDGET ("adjustment_volume"), volume_value_changed_cb, self);
-	gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("adjustment_volume")), volume * 100.0);
-	g_signal_handlers_unblock_by_func(GET_WIDGET ("adjustment_volume"), volume_value_changed_cb, self);
+	if (volume == 0.0)
+		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-muted-symbolic", GTK_ICON_SIZE_BUTTON);
+	else if (volume < 0.33)
+		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-low-symbolic", GTK_ICON_SIZE_BUTTON);
+	else if (volume < 0.66)
+		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-medium-symbolic", GTK_ICON_SIZE_BUTTON);
+	else
+		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-high-symbolic", GTK_ICON_SIZE_BUTTON);
+
+	g_signal_handlers_block_by_func(GET_WIDGET ("volume_adjustment"), volume_value_changed_cb, self);
+	gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("volume_adjustment")), volume * 100.0);
+	g_signal_handlers_unblock_by_func(GET_WIDGET ("volume_adjustment"), volume_value_changed_cb, self);
 }
 
 
@@ -643,8 +672,8 @@ update_play_button (GthMediaViewerPage *self,
 {
 	if (! self->priv->playing && (new_state == GST_STATE_PLAYING)) {
 		set_playing_state (self, TRUE);
-		gtk_image_set_from_stock (GTK_IMAGE (GET_WIDGET ("button_play_image")), GTK_STOCK_MEDIA_PAUSE, GTK_ICON_SIZE_LARGE_TOOLBAR);
-		gtk_widget_set_tooltip_text (GET_WIDGET ("button_play_image"), _("Pause"));
+		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("play_button_image")), "media-playback-pause-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
+		gtk_widget_set_tooltip_text (GET_WIDGET ("play_button_image"), _("Pause"));
 
 		if (self->priv->update_progress_id == 0)
 			self->priv->update_progress_id = gdk_threads_add_timeout (PROGRESS_DELAY, update_progress_cb, self);
@@ -653,8 +682,8 @@ update_play_button (GthMediaViewerPage *self,
 	}
 	else if (self->priv->playing && (new_state != GST_STATE_PLAYING)) {
 		set_playing_state (self, FALSE);
-		gtk_image_set_from_stock (GTK_IMAGE (GET_WIDGET ("button_play_image")), GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_LARGE_TOOLBAR);
-		gtk_widget_set_tooltip_text (GET_WIDGET ("button_play_image"), _("Play"));
+		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("play_button_image")), "media-playback-start-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
+		gtk_widget_set_tooltip_text (GET_WIDGET ("play_button_image"), _("Play"));
 
 		if (self->priv->update_progress_id != 0) {
 			 g_source_remove (self->priv->update_progress_id);
@@ -689,8 +718,6 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 				      self);
 	gtk_ui_manager_insert_action_group (gth_browser_get_ui_manager (browser), self->priv->actions, 0);
 
-	self->priv->area_box = gtk_vbox_new (FALSE, 0);
-
 	/* video area */
 
 	self->priv->area = gtk_drawing_area_new ();
@@ -704,7 +731,6 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 						  | GDK_BUTTON_MOTION_MASK));
 	gtk_widget_set_can_focus (self->priv->area, TRUE);
 	gtk_widget_show (self->priv->area);
-	gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->area, TRUE, TRUE, 0);
 
 	g_signal_connect (G_OBJECT (self->priv->area),
 			  "realize",
@@ -715,8 +741,8 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 			  G_CALLBACK (video_area_unrealize_cb),
 			  self);
 	g_signal_connect (G_OBJECT (self->priv->area),
-			  "expose_event",
-			  G_CALLBACK (video_area_expose_event_cb),
+			  "draw",
+			  G_CALLBACK (video_area_draw_cb),
 			  self);
 	g_signal_connect (G_OBJECT (self->priv->area),
 			  "button_press_event",
@@ -740,23 +766,56 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 	self->priv->builder = _gtk_builder_new_from_file ("mediabar.ui", "gstreamer_tools");
 	self->priv->mediabar = GET_WIDGET ("mediabar");
 	gtk_widget_show (self->priv->mediabar);
+
+	g_signal_connect (GET_WIDGET ("volume_adjustment"),
+			  "value-changed",
+			  G_CALLBACK (volume_value_changed_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("position_adjustment"),
+			  "value-changed",
+			  G_CALLBACK (position_value_changed_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("volume_scale"),
+			  "format-value",
+			  G_CALLBACK (volume_scale_format_value_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("position_scale"),
+			  "change-value",
+			  G_CALLBACK (position_scale_change_value_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("position_scale"),
+			  "button-press-event",
+			  G_CALLBACK (position_scale_button_press_event_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("position_scale"),
+			  "button-release-event",
+			  G_CALLBACK (position_scale_button_release_event_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("play_button"),
+			  "clicked",
+			  G_CALLBACK (play_button_clicked_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("volume_togglebutton"),
+			  "toggled",
+			  G_CALLBACK (volume_togglebutton_toggled_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("play_slower_button"),
+			  "clicked",
+			  G_CALLBACK (play_slower_button_clicked_cb),
+			  self);
+	g_signal_connect (GET_WIDGET ("play_faster_button"),
+			  "clicked",
+			  G_CALLBACK (play_faster_button_clicked_cb),
+			  self);
+
+	self->priv->area_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+	gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->area, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->mediabar, FALSE, FALSE, 0);
-
-	g_signal_connect (GET_WIDGET ("adjustment_volume"), "value-changed", G_CALLBACK (volume_value_changed_cb), self);
-	g_signal_connect (GET_WIDGET ("adjustment_position"), "value-changed", G_CALLBACK (position_value_changed_cb), self);
-	g_signal_connect (GET_WIDGET ("hscale_volume"), "format-value", G_CALLBACK (hscale_volume_format_value_cb), self);
-	g_signal_connect (GET_WIDGET ("hscale_position"), "change-value", G_CALLBACK (hscale_position_change_value_cb), self);
-	g_signal_connect (GET_WIDGET ("hscale_position"), "button-release-event", G_CALLBACK (hscale_position_button_release_event_cb), self);
-	g_signal_connect (GET_WIDGET ("button_play"), "clicked", G_CALLBACK (button_play_clicked_cb), self);
-	g_signal_connect (GET_WIDGET ("togglebutton_volume"), "toggled", G_CALLBACK (togglebutton_volume_toggled_cb), self);
-	g_signal_connect (GET_WIDGET ("button_play_slower"), "clicked", G_CALLBACK (button_play_slower_clicked_cb), self);
-	g_signal_connect (GET_WIDGET ("button_play_faster"), "clicked", G_CALLBACK (button_play_faster_clicked_cb), self);
-
 	gtk_widget_show (self->priv->area_box);
 	gth_browser_set_viewer_widget (browser, self->priv->area_box);
 
-	gtk_widget_realize (self->priv->area);
-	gdk_window_ensure_native (gtk_widget_get_window (self->priv->area));
+	gtk_widget_realize (self->priv->area_box);
+
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 }
 
@@ -818,8 +877,6 @@ set_playbin_window (GstBus             *bus,
 		    GstMessage         *message,
 		    GthMediaViewerPage *self)
 {
-	GstXOverlay *image_sink;
-
 	/* ignore anything but 'prepare-xwindow-id' element messages */
 
 	if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
@@ -827,10 +884,17 @@ set_playbin_window (GstBus             *bus,
 	if (! gst_structure_has_name (message->structure, "prepare-xwindow-id"))
 		return GST_BUS_PASS;
 
-	image_sink = GST_X_OVERLAY (GST_MESSAGE_SRC (message));
-	gst_x_overlay_set_xwindow_id (image_sink, GDK_WINDOW_XID (gtk_widget_get_window (self->priv->area)));
-	g_object_set (image_sink, "force-aspect-ratio", TRUE, NULL);
-	self->priv->xwin_assigned = TRUE;
+	if (self->priv->video_window_xid != 0) {
+		GstXOverlay *xoverlay;
+
+		xoverlay = GST_X_OVERLAY (GST_MESSAGE_SRC (message));
+		gst_x_overlay_set_window_handle (xoverlay, self->priv->video_window_xid);
+		self->priv->xwin_assigned = TRUE;
+
+		g_object_set (xoverlay, "force-aspect-ratio", TRUE, NULL);
+	}
+	else
+		g_warning ("Should have obtained video_window_xid by now!");
 
 	gst_message_unref (message);
 
@@ -896,9 +960,16 @@ update_stream_info (GthMediaViewerPage *self)
 
 		if ((caps = gst_pad_get_negotiated_caps (videopad)) != NULL) {
 			GstStructure *structure;
+			int           video_width;
+			int           video_height;
 
 			structure = gst_caps_get_structure (caps, 0);
 			gst_structure_get_fraction (structure, "framerate", &self->priv->video_fps_n, &self->priv->video_fps_d);
+			gst_structure_get_int (structure, "width", &video_width);
+			gst_structure_get_int (structure, "height", &video_height);
+
+			g_file_info_set_attribute_int32 (self->priv->file_data->info, "frame::width", video_width);
+			g_file_info_set_attribute_int32 (self->priv->file_data->info, "frame::height", video_height);
 
 			gst_caps_unref (caps);
 		}
@@ -934,9 +1005,15 @@ bus_message_cb (GstBus     *bus,
 
 		update_current_position_bar (self, TRUE);
 
+		if ((old_state == GST_STATE_NULL) && (new_state == GST_STATE_READY) && (pending_state != GST_STATE_PAUSED)) {
+			update_stream_info (self);
+			gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (self));
+			gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), self->priv->file_data, TRUE);
+		}
 		if ((old_state == GST_STATE_READY) && (new_state == GST_STATE_PAUSED)) {
 			update_stream_info (self);
 			gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (self));
+			gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), self->priv->file_data, TRUE);
 		}
 		if ((old_state == GST_STATE_READY) || (new_state == GST_STATE_PAUSED))
 			update_volume_from_playbin (self);
@@ -961,7 +1038,11 @@ bus_message_cb (GstBus     *bus,
 	case GST_MESSAGE_BUFFERING: {
 		int percent = 0;
 		gst_message_parse_buffering (message, &percent);
-		g_print ("Buffering (%%%u percent done)", percent);
+		/* g_print ("Buffering (%%%u percent done)", percent); */
+		break;
+
+	case GST_MESSAGE_ERROR:
+		gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), self->priv->file_data, FALSE);
 		break;
 	}
 
@@ -981,25 +1062,9 @@ playbin_notify_volume_cb (GObject    *playbin,
 
 
 static void
-gth_media_viewer_page_real_show (GthViewerPage *base)
+create_playbin (GthMediaViewerPage *self)
 {
-	GthMediaViewerPage *self;
-	GError             *error = NULL;
-	GstBus             *bus;
-	char               *uri;
-
-	self = (GthMediaViewerPage*) base;
-
-	if (self->priv->merge_id != 0)
-		return;
-
-	self->priv->merge_id = gtk_ui_manager_add_ui_from_string (gth_browser_get_ui_manager (self->priv->browser), media_viewer_ui_info, -1, &error);
-	if (self->priv->merge_id == 0) {
-		g_warning ("ui building failed: %s", error->message);
-		g_error_free (error);
-	}
-
-	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
+	GstBus *bus;
 
 	if (self->priv->playbin != NULL)
 		return;
@@ -1013,14 +1078,36 @@ gth_media_viewer_page_real_show (GthViewerPage *base)
 	gst_bus_add_signal_watch (bus);
 	g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
 
-	if (self->priv->file_data == NULL)
-		return;
+}
 
-	uri = g_file_get_uri (self->priv->file_data->file);
-	g_object_set (G_OBJECT (self->priv->playbin), "uri", uri, NULL);
-	gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
 
-	g_free (uri);
+static void
+gth_media_viewer_page_real_show (GthViewerPage *base)
+{
+	GthMediaViewerPage *self = GTH_MEDIA_VIEWER_PAGE (base);
+	GError             *error = NULL;
+
+	self->priv->visible = TRUE;
+
+	if (self->priv->merge_id == 0) {
+		self->priv->merge_id = gtk_ui_manager_add_ui_from_string (gth_browser_get_ui_manager (self->priv->browser), media_viewer_ui_info, -1, &error);
+		if (self->priv->merge_id == 0) {
+			g_warning ("ui building failed: %s", error->message);
+			g_error_free (error);
+		}
+	}
+	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
+
+	create_playbin (self);
+	if (self->priv->file_data != NULL) {
+		char *uri;
+
+		uri = g_file_get_uri (self->priv->file_data->file);
+		g_object_set (G_OBJECT (self->priv->playbin), "uri", uri, NULL);
+		gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+
+		g_free (uri);
+	}
 }
 
 
@@ -1031,6 +1118,8 @@ gth_media_viewer_page_real_hide (GthViewerPage *base)
 
 	self = (GthMediaViewerPage*) base;
 
+	self->priv->visible = FALSE;
+
 	if (self->priv->merge_id != 0) {
 		gtk_ui_manager_remove_ui (gth_browser_get_ui_manager (self->priv->browser), self->priv->merge_id);
 		self->priv->merge_id = 0;
@@ -1058,7 +1147,7 @@ set_to_paused (gpointer user_data)
 
 	self->priv->set_to_pause_id = 0;
 	if (self->priv->playbin != NULL)
-		gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
+		gst_element_set_state (self->priv->playbin, self->priv->visible ? GST_STATE_PLAYING : GST_STATE_READY);
 
 	return FALSE;
 }
@@ -1102,12 +1191,12 @@ gth_media_viewer_page_real_view (GthViewerPage *base,
 
 	/**/
 
-	gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self), self->priv->file_data, TRUE);
-	g_signal_handlers_block_by_func(GET_WIDGET ("adjustment_position"), position_value_changed_cb, self);
-	gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("adjustment_position")), 0.0);
-	g_signal_handlers_unblock_by_func(GET_WIDGET ("adjustment_position"), position_value_changed_cb, self);
+	g_signal_handlers_block_by_func(GET_WIDGET ("position_adjustment"), position_value_changed_cb, self);
+	gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("position_adjustment")), 0.0);
+	g_signal_handlers_unblock_by_func(GET_WIDGET ("position_adjustment"), position_value_changed_cb, self);
 	reset_player_state (self);
 
+	create_playbin (self);
 	if (self->priv->playbin == NULL)
 		return;
 
@@ -1214,9 +1303,6 @@ gth_media_viewer_page_real_update_sensitivity (GthViewerPage *base)
 {
 	GthMediaViewerPage *self = (GthMediaViewerPage *) base;
 
-	/*gtk_widget_set_sensitive (GET_WIDGET ("button_play_slower"), self->priv->playing);
-	gtk_widget_set_sensitive (GET_WIDGET ("button_play_faster"), self->priv->playing);*/
-
 	gtk_widget_set_sensitive (GET_WIDGET ("volume_box"), self->priv->has_audio);
 	set_action_sensitive (self, "MediaViewer_Screenshot", self->priv->has_video);
 }
@@ -1268,6 +1354,20 @@ gth_media_viewer_page_real_update_info (GthViewerPage *base,
 }
 
 
+static void
+gth_media_viewer_page_real_shrink_wrap (GthViewerPage *base,
+					gboolean       value,
+					int           *other_width,
+					int           *other_height)
+{
+	GthMediaViewerPage *self = GTH_MEDIA_VIEWER_PAGE (base);
+
+	if (! value || (other_width == NULL) || (other_height == NULL))
+		return;
+	*other_height += _gtk_widget_get_allocated_height (self->priv->mediabar);
+}
+
+
 static void
 gth_media_viewer_page_finalize (GObject *obj)
 {
@@ -1304,7 +1404,6 @@ gth_media_viewer_page_finalize (GObject *obj)
 static void
 gth_media_viewer_page_class_init (GthMediaViewerPageClass *klass)
 {
-	gth_media_viewer_page_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (GthMediaViewerPagePrivate));
 
 	G_OBJECT_CLASS (klass)->finalize = gth_media_viewer_page_finalize;
@@ -1312,7 +1411,7 @@ gth_media_viewer_page_class_init (GthMediaViewerPageClass *klass)
 
 
 static void
-gth_viewer_page_interface_init (GthViewerPageIface *iface)
+gth_viewer_page_interface_init (GthViewerPageInterface *iface)
 {
 	iface->activate = gth_media_viewer_page_real_activate;
 	iface->deactivate = gth_media_viewer_page_real_deactivate;
@@ -1329,15 +1428,17 @@ gth_viewer_page_interface_init (GthViewerPageIface *iface)
 	iface->save_as = gth_media_viewer_page_real_save_as;
 	iface->revert = gth_media_viewer_page_real_revert;
 	iface->update_info = gth_media_viewer_page_real_update_info;
+	iface->shrink_wrap = gth_media_viewer_page_real_shrink_wrap;
 }
 
 
 static void
-gth_media_viewer_page_instance_init (GthMediaViewerPage *self)
+gth_media_viewer_page_init (GthMediaViewerPage *self)
 {
-	self->priv = GTH_MEDIA_VIEWER_PAGE_GET_PRIVATE (self);
+	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_MEDIA_VIEWER_PAGE, GthMediaViewerPagePrivate);
 	self->priv->update_progress_id = 0;
 	self->priv->set_to_pause_id = 0;
+	self->priv->video_window_xid = 0;
 	self->priv->xwin_assigned = FALSE;
 	self->priv->has_video = FALSE;
 	self->priv->has_audio = FALSE;
@@ -1346,34 +1447,7 @@ gth_media_viewer_page_instance_init (GthMediaViewerPage *self)
 	self->priv->icon = NULL;
 	self->priv->cursor_visible = TRUE;
 	self->priv->screensaver = gth_screensaver_new (NULL);
-}
-
-
-GType
-gth_media_viewer_page_get_type (void) {
-	static GType gth_media_viewer_page_type_id = 0;
-	if (gth_media_viewer_page_type_id == 0) {
-		static const GTypeInfo g_define_type_info = {
-			sizeof (GthMediaViewerPageClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) gth_media_viewer_page_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (GthMediaViewerPage),
-			0,
-			(GInstanceInitFunc) gth_media_viewer_page_instance_init,
-			NULL
-		};
-		static const GInterfaceInfo gth_viewer_page_info = {
-			(GInterfaceInitFunc) gth_viewer_page_interface_init,
-			(GInterfaceFinalizeFunc) NULL,
-			NULL
-		};
-		gth_media_viewer_page_type_id = g_type_register_static (G_TYPE_OBJECT, "GthMediaViewerPage", &g_define_type_info, 0);
-		g_type_add_interface_static (gth_media_viewer_page_type_id, GTH_TYPE_VIEWER_PAGE, &gth_viewer_page_info);
-	}
-	return gth_media_viewer_page_type_id;
+	self->priv->visible = FALSE;
 }
 
 
