@@ -26,8 +26,10 @@
 
 #define DEFINE_ACTION(x) void x (GtkAction *action, gpointer data);
 
-void gth_browser_activate_action_add_to_selection (GthBrowser *browser,
-						   int         n_selection);
+void gth_browser_activate_action_add_to_selection      (GthBrowser *browser,
+						        int         n_selection);
+void gth_browser_activate_action_remove_from_selection (GthBrowser *browser,
+						        int         n_selection);
 
 DEFINE_ACTION(gth_browser_activate_action_go_selection_1)
 DEFINE_ACTION(gth_browser_activate_action_go_selection_2)
@@ -35,5 +37,6 @@ DEFINE_ACTION(gth_browser_activate_action_go_selection_3)
 DEFINE_ACTION(gth_browser_activate_action_add_to_selection_1)
 DEFINE_ACTION(gth_browser_activate_action_add_to_selection_2)
 DEFINE_ACTION(gth_browser_activate_action_add_to_selection_3)
+DEFINE_ACTION(gth_browser_activate_action_selection_go_to_container)
 
 #endif /* ACTIONS_H */
