@@ -86,7 +86,7 @@ struct _GthTagsEntryPrivate {
 static guint signals[LAST_SIGNAL] = { 0 };
 
 
-G_DEFINE_TYPE (GthTagsEntry, gth_tags_entry, GTK_TYPE_VBOX)
+G_DEFINE_TYPE (GthTagsEntry, gth_tags_entry, GTK_TYPE_BOX)
 
 
 static void
@@ -545,15 +545,6 @@ update_completion_list (GthTagsEntry *self)
 	g_strfreev (self->priv->tags);
 	self->priv->tags = g_strdupv (gth_tags_file_get_tags (tags));
 
-	for (i = 0; self->priv->tags[i] != NULL; i++) {
-		GtkTreeIter iter;
-
-		gtk_list_store_append (self->priv->completion_store, &iter);
-		gtk_list_store_set (self->priv->completion_store, &iter,
-				    COMPLETION_NAME_COLUMN, self->priv->tags[i],
-				    -1);
-	}
-
 	gtk_list_store_clear (self->priv->completion_store);
 	for (i = 0; self->priv->tags[i] != NULL; i++) {
 		GtkTreeIter iter;
@@ -697,6 +688,7 @@ gth_tags_entry_init (GthTagsEntry *self)
 	GtkCellRenderer   *renderer;
 
 	gtk_widget_set_can_focus (GTK_WIDGET (self), TRUE);
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_VERTICAL);
 
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_TAGS_ENTRY, GthTagsEntryPrivate);
 	self->priv->expanded_list.last_used = g_new0 (char *, 1);
@@ -885,6 +877,7 @@ gth_tags_entry_set_tags_from_text (GthTagsEntry *self,
 				   const char   *text)
 {
 	char **tags;
+	int    i;
 
 	if ((text == NULL) || (strcmp (text, "") == 0)) {
 		gth_tags_entry_set_tags (self, NULL);
@@ -892,6 +885,8 @@ gth_tags_entry_set_tags_from_text (GthTagsEntry *self,
 	}
 
 	tags = g_strsplit (text, ",", -1);
+	for (i = 0; tags[i] != NULL; i++)
+		tags[i] = g_strstrip (tags[i]);
 	gth_tags_entry_set_tags (self, tags);
 
 	g_strfreev (tags);
